@@ -142,6 +142,23 @@ on_page_changed (GtkNotebook   *notebook,
   gtk_header_bar_set_subtitle (self->header_bar, subtitle);
 }
 
+static void
+on_titlebar_animation (ExampleWindow *self,
+                       GParamSpec    *pspec)
+{
+  g_autoptr(GEnumClass) enum_class = NULL;
+  DzlTitlebarAnimation mode;
+  GEnumValue *value;
+
+  g_assert (EXAMPLE_IS_WINDOW (self));
+
+  mode = dzl_application_window_get_titlebar_animation (DZL_APPLICATION_WINDOW (self));
+
+  enum_class = g_type_class_ref (DZL_TYPE_TITLEBAR_ANIMATION);
+  value = g_enum_get_value (enum_class, mode);
+  g_print ("Titlebar Animation: %s\n", value->value_nick);
+}
+
 static void
 example_window_class_init (ExampleWindowClass *klass)
 {
@@ -164,6 +181,11 @@ example_window_init (ExampleWindow *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  g_signal_connect (self,
+                    "notify::titlebar-animation",
+                    G_CALLBACK (on_titlebar_animation),
+                    NULL);
+
   g_signal_connect (self->notebook,
                     "notify::page",
                     G_CALLBACK (on_page_changed),
