@@ -148,7 +148,8 @@ create_variant_type (const GType *types,
               break;
             }
 
-          return FALSE;
+          g_string_free (str, TRUE);
+          return NULL;
         }
     }
 
@@ -171,6 +172,8 @@ create_variant_type (const GType *types,
       g_hash_table_insert (cached_types, type_str, type_str);
       ret = (const GVariantType *)type_str;
     }
+  else
+    g_string_free (str, TRUE);
 
   return ret;
 }
@@ -339,7 +342,7 @@ dzl_widget_action_group_finalize (GObject *object)
 {
   DzlWidgetActionGroup *self = (DzlWidgetActionGroup *)object;
 
-  dzl_clear_pointer (&self->enabled, g_hash_table_unref);
+  g_clear_pointer (&self->enabled, g_hash_table_unref);
 
   G_OBJECT_CLASS (dzl_widget_action_group_parent_class)->finalize (object);
 }
