@@ -244,12 +244,14 @@ dzl_file_transfer_set_flags (DzlFileTransfer      *self,
 {
   DzlFileTransferPrivate *priv = dzl_file_transfer_get_instance_private (self);
 
+  DZL_ENTRY;
+
   g_return_if_fail (DZL_IS_FILE_TRANSFER (self));
 
   if (priv->executed)
     {
       g_warning ("Cannot set flags after executing transfer");
-      return;
+      DZL_EXIT;
     }
 
   if (priv->flags != flags)
@@ -257,6 +259,8 @@ dzl_file_transfer_set_flags (DzlFileTransfer      *self,
       priv->flags = flags;
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_FLAGS]);
     }
+
+  DZL_EXIT;
 }
 
 gdouble
@@ -279,18 +283,20 @@ file_walk_full (GFile            *parent,
                 FileWalkCallback  callback,
                 gpointer          user_data)
 {
+  DZL_ENTRY;
+
   g_assert (!parent || G_IS_FILE (parent));
   g_assert (G_IS_FILE_INFO (info));
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
   g_assert (callback != NULL);
 
   if (g_cancellable_is_cancelled (cancellable))
-    return;
+    DZL_EXIT;
 
   callback (parent, info, user_data);
 
   if (g_file_info_get_is_symlink (info))
-    return;
+    DZL_EXIT;
 
   if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
     {
@@ -299,7 +305,7 @@ file_walk_full (GFile            *parent,
       const gchar *name = g_file_info_get_name (info);
 
       if (name == NULL)
-        return;
+        DZL_EXIT;
 
       child = g_file_get_child (parent, name);
       enumerator = g_file_enumerate_children (child, QUERY_ATTRS, QUERY_FLAGS, cancellable, NULL);
@@ -308,15 +314,18 @@ file_walk_full (GFile            *parent,
         {
           gpointer infoptr;
 
-          while (NULL != (infoptr = g_file_enumerator_next_file (enumerator, cancellable, NULL)))
+          while ((infoptr = g_file_enumerator_next_file (enumerator, cancellable, NULL)))
             {
               g_autoptr(GFileInfo) grandchild_info = infoptr;
+
               file_walk_full (child, grandchild_info, cancellable, callback, user_data);
             }
 
           g_file_enumerator_close (enumerator, cancellable, NULL);
         }
     }
+
+  DZL_EXIT;
 }
 
 static void
@@ -328,6 +337,8 @@ file_walk (GFile            *root,
   g_autoptr(GFile) parent = NULL;
   g_autoptr(GFileInfo) info = NULL;
 
+  DZL_ENTRY;
+
   g_assert (G_IS_FILE (root));
   g_assert (callback != NULL);
 
@@ -338,6 +349,8 @@ file_walk (GFile            *root,
   info = g_file_query_info (root, QUERY_ATTRS, QUERY_FLAGS, cancellable, NULL);
   if (info != NULL)
     file_walk_full (parent, info, cancellable, callback, user_data);
+
+  DZL_EXIT;
 }
 
 static void
@@ -348,6 +361,8 @@ handle_preflight_cb (GFile     *file,
   DzlFileTransferStat *stat_buf = user_data;
   GFileType file_type;
 
+  DZL_ENTRY;
+
   g_assert (G_IS_FILE (file));
   g_assert (G_IS_FILE_INFO (child_info));
   g_assert (stat_buf != NULL);
@@ -363,6 +378,8 @@ handle_preflight_cb (GFile     *file,
       stat_buf->n_files_total++;
       stat_buf->n_bytes_total += g_file_info_get_size (child_info);
     }
+
+  DZL_EXIT;
 }
 
 static void
@@ -372,11 +389,13 @@ handle_preflight (DzlFileTransfer     *self,
 {
   DzlFileTransferPrivate *priv = dzl_file_transfer_get_instance_private (self);
 
+  DZL_ENTRY;
+
   g_assert (DZL_IS_FILE_TRANSFER (self));
   g_assert (opers != NULL);
 
   if (g_cancellable_is_cancelled (cancellable))
-    return;
+    DZL_EXIT;
 
   for (guint i = 0; i < opers->len; i++)
     {
@@ -392,6 +411,8 @@ handle_preflight (DzlFileTransfer     *self,
       if (oper->error != NULL)
         break;
     }
+
+  DZL_EXIT;
 }
 
 static void
@@ -417,6 +438,8 @@ handle_copy_cb (GFile     *file,
   Oper *oper = user_data;
   GFileType file_type;
 
+  DZL_ENTRY;
+
   g_assert (DZL_IS_FILE_TRANSFER (oper->self));
   g_assert (G_IS_FILE (oper->src));
   g_assert (G_IS_FILE (oper->dst));
@@ -424,10 +447,10 @@ handle_copy_cb (GFile     *file,
   g_assert (G_IS_FILE_INFO (child_info));
 
   if (oper->error != NULL)
-    return;
+    DZL_EXIT;
 
   if (g_cancellable_is_cancelled (oper->cancellable))
-    return;
+    DZL_EXIT;
 
   priv = dzl_file_transfer_get_instance_private (oper->self);
 
@@ -435,7 +458,7 @@ handle_copy_cb (GFile     *file,
   name = g_file_info_get_name (child_info);
 
   if (name == NULL)
-    return;
+    DZL_EXIT;
 
   src = g_file_get_child (file, name);
 
@@ -485,6 +508,8 @@ handle_copy_cb (GFile     *file,
     default:
       break;
     }
+
+  DZL_EXIT;
 }
 
 static void
@@ -492,12 +517,14 @@ handle_copy (DzlFileTransfer *self,
              GPtrArray       *opers,
              GCancellable    *cancellable)
 {
+  DZL_ENTRY;
+
   g_assert (DZL_IS_FILE_TRANSFER (self));
   g_assert (opers != NULL);
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   if (g_cancellable_is_cancelled (cancellable))
-    return;
+    DZL_EXIT;
 
   for (guint i = 0; i < opers->len; i++)
     {
@@ -518,6 +545,8 @@ handle_copy (DzlFileTransfer *self,
             break;
         }
     }
+
+  DZL_EXIT;
 }
 
 static void
@@ -527,12 +556,14 @@ handle_removal (DzlFileTransfer *self,
 {
   g_autoptr(DzlDirectoryReaper) reaper = NULL;
 
+  DZL_ENTRY;
+
   g_assert (DZL_IS_FILE_TRANSFER (self));
   g_assert (opers != NULL);
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   if (g_cancellable_is_cancelled (cancellable))
-    return;
+    DZL_EXIT;
 
   reaper = dzl_directory_reaper_new ();
 
@@ -546,7 +577,7 @@ handle_removal (DzlFileTransfer *self,
 
       /* Don't delete anything if there was a failure */
       if (oper->error != NULL)
-        return;
+        DZL_EXIT;
 
       if (g_file_query_file_type (oper->src, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) == G_FILE_TYPE_DIRECTORY)
         dzl_directory_reaper_add_directory (reaper, oper->src, 0);
@@ -555,6 +586,22 @@ handle_removal (DzlFileTransfer *self,
     }
 
   dzl_directory_reaper_execute (reaper, cancellable, NULL);
+
+  DZL_EXIT;
+}
+
+static gboolean
+dzl_file_transfer_do_notify_progress (gpointer data)
+{
+  DzlFileTransfer *self = data;
+
+  DZL_ENTRY;
+
+  g_assert (DZL_IS_FILE_TRANSFER (self));
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_PROGRESS]);
+
+  DZL_RETURN (G_SOURCE_CONTINUE);
 }
 
 static void
@@ -566,6 +613,7 @@ dzl_file_transfer_worker (GTask        *task,
   DzlFileTransfer *self = source_object;
   DzlFileTransferPrivate *priv = dzl_file_transfer_get_instance_private (self);
   GPtrArray *opers = task_data;
+  guint notify_source;
 
   DZL_ENTRY;
 
@@ -574,7 +622,11 @@ dzl_file_transfer_worker (GTask        *task,
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
   g_assert (opers != NULL);
 
-  /* TODO: Start GSource for notifies */
+  notify_source = g_timeout_add_full (G_PRIORITY_LOW,
+                                      1000 / 4, /* 4x a second */
+                                      dzl_file_transfer_do_notify_progress,
+                                      g_object_ref (self),
+                                      g_object_unref);
 
   for (guint i = 0; i < opers->len; i++)
     {
@@ -597,13 +649,14 @@ dzl_file_transfer_worker (GTask        *task,
       if (oper->error != NULL)
         {
           g_task_return_error (task, g_steal_pointer (&oper->error));
-          DZL_EXIT;
+          DZL_GOTO (cleanup);
         }
     }
 
   g_task_return_boolean (task, TRUE);
 
-  /* TODO: Stop GSource for notifies */
+cleanup:
+  g_source_remove (notify_source);
 
   DZL_EXIT;
 }
@@ -628,10 +681,19 @@ dzl_file_transfer_execute (DzlFileTransfer  *self,
 
   if (priv->executed)
     {
-      g_task_return_new_error (task,
-                               G_IO_ERROR,
-                               G_IO_ERROR_INVAL,
-                               "Transfer can only be executed once.");
+      g_set_error (error,
+                   G_IO_ERROR,
+                   G_IO_ERROR_INVAL,
+                   "Transfer can only be executed once.");
+      DZL_RETURN (FALSE);
+    }
+
+  if (priv->opers->len == 0)
+    {
+      g_set_error (error,
+                   G_IO_ERROR,
+                   G_IO_ERROR_INVAL,
+                   "Transfer can only be executed once.");
       DZL_RETURN (FALSE);
     }
 
@@ -661,7 +723,7 @@ dzl_file_transfer_execute_async (DzlFileTransfer     *self,
   g_return_if_fail (DZL_IS_FILE_TRANSFER (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  task = g_task_new (self, cancellable, NULL, NULL);
+  task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, dzl_file_transfer_execute);
 
   if (priv->executed)
@@ -675,6 +737,15 @@ dzl_file_transfer_execute_async (DzlFileTransfer     *self,
 
   priv->executed = TRUE;
 
+  if (priv->opers->len == 0)
+    {
+      g_task_return_new_error (task,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVAL,
+                               "No transfers were provided to execute");
+      DZL_EXIT;
+    }
+
   g_task_set_check_cancellable (task, TRUE);
   g_task_set_return_on_cancel (task, TRUE);
   g_task_set_priority (task, io_priority);
