@@ -21,7 +21,9 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <stdlib.h>
-#include <unistd.h>
+#ifdef __linux__
+# include <unistd.h>
+#endif
 #include <stdio.h>
 #if defined(__FreeBSD__)
 # include <errno.h>
@@ -330,13 +332,13 @@ dzl_cpu_model_finalize (GObject *object)
   DzlCpuModel *self = (DzlCpuModel *)object;
 
 #ifdef __linux__
-  dzl_clear_pointer (&self->stat_buf, g_free);
+  g_clear_pointer (&self->stat_buf, g_free);
   if (self->stat_fd != -1)
     close (self->stat_fd);
 #endif
 
   dzl_clear_source (&self->poll_source);
-  dzl_clear_pointer (&self->cpu_info, g_array_unref);
+  g_clear_pointer (&self->cpu_info, g_array_unref);
 
   G_OBJECT_CLASS (dzl_cpu_model_parent_class)->finalize (object);
 }
