@@ -108,6 +108,8 @@ dzl_dock_manager_do_set_focus (DzlDockManager *self,
       g_clear_object (&priv->grab);
     }
 
+  g_assert (priv->grab == NULL);
+
   /* Start the grab process */
   if (grab != NULL)
     {
@@ -268,7 +270,9 @@ dzl_dock_manager_finalize (GObject *object)
   DzlDockManager *self = (DzlDockManager *)object;
   DzlDockManagerPrivate *priv = dzl_dock_manager_get_instance_private (self);
 
-  dzl_clear_pointer (&priv->queued_focus_by_toplevel, g_hash_table_unref);
+  g_clear_object (&priv->grab);
+
+  g_clear_pointer (&priv->queued_focus_by_toplevel, g_hash_table_unref);
 
   dzl_clear_source (&priv->queued_handler);
 
@@ -280,7 +284,7 @@ dzl_dock_manager_finalize (GObject *object)
       g_ptr_array_remove_index (priv->docks, priv->docks->len - 1);
     }
 
-  dzl_clear_pointer (&priv->docks, g_ptr_array_unref);
+  g_clear_pointer (&priv->docks, g_ptr_array_unref);
 
   G_OBJECT_CLASS (dzl_dock_manager_parent_class)->finalize (object);
 }
