@@ -27,7 +27,7 @@
 
 typedef struct
 {
-  GtkPositionType child_edge : 2;
+  GtkPositionType child_edge;
 } DzlDockPanedPrivate;
 
 G_DEFINE_TYPE_EXTENDED (DzlDockPaned, dzl_dock_paned, DZL_TYPE_MULTI_PANED, 0,
@@ -49,7 +49,12 @@ dzl_dock_paned_add (GtkContainer *container,
   GTK_CONTAINER_CLASS (dzl_dock_paned_parent_class)->add (container, widget);
 
   if (DZL_IS_DOCK_ITEM (widget))
-    dzl_dock_item_adopt (DZL_DOCK_ITEM (self), DZL_DOCK_ITEM (widget));
+    {
+      dzl_dock_item_adopt (DZL_DOCK_ITEM (self), DZL_DOCK_ITEM (widget));
+
+      /* Always available, so emit presented */
+      dzl_dock_item_emit_presented (DZL_DOCK_ITEM (widget));
+    }
 }
 
 static void
