@@ -38,3 +38,9 @@ static void
 dzl_preferences_flow_box_init (DzlPreferencesFlowBox *self)
 {
 }
+
+GtkWidget *
+dzl_preferences_flow_box_new (void)
+{
+  return g_object_new (DZL_TYPE_PREFERENCES_FLOW_BOX, NULL);
+}
