@@ -105,12 +105,13 @@ dzl_fuzzy_index_finalize (GObject *object)
 {
   DzlFuzzyIndex *self = (DzlFuzzyIndex *)object;
 
-  dzl_clear_pointer (&self->mapped_file, g_mapped_file_unref);
-  dzl_clear_pointer (&self->variant, g_variant_unref);
-  dzl_clear_pointer (&self->documents, g_variant_unref);
-  dzl_clear_pointer (&self->keys, g_variant_unref);
-  dzl_clear_pointer (&self->tables, g_variant_dict_unref);
-  dzl_clear_pointer (&self->lookaside, g_variant_unref);
+  g_clear_pointer (&self->mapped_file, g_mapped_file_unref);
+  g_clear_pointer (&self->variant, g_variant_unref);
+  g_clear_pointer (&self->documents, g_variant_unref);
+  g_clear_pointer (&self->keys, g_variant_unref);
+  g_clear_pointer (&self->tables, g_variant_dict_unref);
+  g_clear_pointer (&self->lookaside, g_variant_unref);
+  g_clear_pointer (&self->metadata, g_variant_dict_unref);
 
   G_OBJECT_CLASS (dzl_fuzzy_index_parent_class)->finalize (object);
 }
