@@ -81,8 +81,8 @@ static void
 load_state_frame_free (LoadStateFrame *frm)
 {
   g_clear_object (&frm->context);
-  dzl_clear_pointer (&frm->accelerator, g_free);
-  dzl_clear_pointer (&frm->signal, g_free);
+  g_clear_pointer (&frm->accelerator, g_free);
+  g_clear_pointer (&frm->signal, g_free);
 
   g_slist_free_full (frm->params, g_free);
   frm->params = NULL;
