@@ -31,14 +31,18 @@ typedef struct
 
   /* interned string */
   const gchar *icon_name;
+  const gchar *secondary_icon_name;
 
   GIcon *icon;
+  GIcon *secondary_icon;
 } DzlSuggestionPrivate;
 
 enum {
   PROP_0,
   PROP_ICON_NAME,
   PROP_ICON,
+  PROP_SECONDARY_ICON_NAME,
+  PROP_SECONDARY_ICON,
   PROP_ID,
   PROP_SUBTITLE,
   PROP_TITLE,
@@ -69,6 +73,19 @@ dzl_suggestion_real_get_icon (DzlSuggestion *self)
   return NULL;
 }
 
+static GIcon *
+dzl_suggestion_real_get_secondary_icon (DzlSuggestion *self)
+{
+  DzlSuggestionPrivate *priv = dzl_suggestion_get_instance_private (self);
+
+  g_assert (DZL_IS_SUGGESTION (self));
+
+  if (priv->secondary_icon_name != NULL)
+    return g_icon_new_for_string (priv->secondary_icon_name, NULL);
+
+  return NULL;
+}
+
 static void
 dzl_suggestion_finalize (GObject *object)
 {
@@ -77,9 +94,9 @@ dzl_suggestion_finalize (GObject *object)
 
   priv->icon_name = NULL;
 
-  dzl_clear_pointer (&priv->title, g_free);
-  dzl_clear_pointer (&priv->subtitle, g_free);
-  dzl_clear_pointer (&priv->id, g_free);
+  g_clear_pointer (&priv->title, g_free);
+  g_clear_pointer (&priv->subtitle, g_free);
+  g_clear_pointer (&priv->id, g_free);
 
   G_OBJECT_CLASS (dzl_suggestion_parent_class)->finalize (object);
 }
@@ -106,6 +123,14 @@ dzl_suggestion_get_property (GObject    *object,
       g_value_take_object (value, dzl_suggestion_get_icon (self));
       break;
 
+    case PROP_SECONDARY_ICON_NAME:
+      g_value_set_static_string (value, dzl_suggestion_get_secondary_icon_name (self));
+      break;
+
+    case PROP_SECONDARY_ICON:
+      g_value_take_object (value, dzl_suggestion_get_secondary_icon (self));
+      break;
+
     case PROP_TITLE:
       g_value_set_string (value, dzl_suggestion_get_title (self));
       break;
@@ -133,6 +158,10 @@ dzl_suggestion_set_property (GObject      *object,
       dzl_suggestion_set_icon_name (self, g_value_get_string (value));
       break;
 
+    case PROP_SECONDARY_ICON_NAME:
+      dzl_suggestion_set_secondary_icon_name (self, g_value_get_string (value));
+      break;
+
     case PROP_ID:
       dzl_suggestion_set_id (self, g_value_get_string (value));
       break;
@@ -160,6 +189,7 @@ dzl_suggestion_class_init (DzlSuggestionClass *klass)
   object_class->set_property = dzl_suggestion_set_property;
 
   klass->get_icon = dzl_suggestion_real_get_icon;
+  klass->get_secondary_icon = dzl_suggestion_real_get_secondary_icon;
 
   properties [PROP_ID] =
     g_param_spec_string ("id",
@@ -182,6 +212,20 @@ dzl_suggestion_class_init (DzlSuggestionClass *klass)
                          NULL,
                          (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
+  properties [PROP_SECONDARY_ICON] =
+    g_param_spec_object ("secondary-icon",
+                         "Secondary Icon",
+                         "The secondary GIcon for the suggestion on the right",
+                         G_TYPE_ICON,
+                         (G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
+
+  properties [PROP_SECONDARY_ICON_NAME] =
+    g_param_spec_string ("secondary-icon-name",
+                         "Secondary Icon Name",
+                         "The name of the secondary icon to display",
+                         NULL,
+                         (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
+
   properties [PROP_TITLE] =
     g_param_spec_string ("title",
                          "Title",
@@ -240,6 +284,16 @@ dzl_suggestion_get_icon_name (DzlSuggestion *self)
   return priv->icon_name;
 }
 
+const gchar *
+dzl_suggestion_get_secondary_icon_name (DzlSuggestion *self)
+{
+  DzlSuggestionPrivate *priv = dzl_suggestion_get_instance_private (self);
+
+  g_return_val_if_fail (DZL_IS_SUGGESTION (self), NULL);
+
+  return priv->secondary_icon_name;
+}
+
 const gchar *
 dzl_suggestion_get_title (DzlSuggestion *self)
 {
@@ -277,6 +331,23 @@ dzl_suggestion_set_icon_name (DzlSuggestion *self,
     }
 }
 
+void
+dzl_suggestion_set_secondary_icon_name (DzlSuggestion *self,
+                                        const gchar   *icon_name)
+{
+  DzlSuggestionPrivate *priv = dzl_suggestion_get_instance_private (self);
+
+  g_return_if_fail (DZL_IS_SUGGESTION (self));
+
+  icon_name = g_intern_string (icon_name);
+
+  if (priv->secondary_icon_name != icon_name)
+    {
+      priv->secondary_icon_name = icon_name;
+      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SECONDARY_ICON_NAME]);
+    }
+}
+
 void
 dzl_suggestion_set_id (DzlSuggestion *self,
                        const gchar   *id)
@@ -431,3 +502,49 @@ dzl_suggestion_get_icon_surface (DzlSuggestion *self,
 
   return NULL;
 }
+
+/**
+ * dzl_suggestion_get_secondary_icon:
+ * @self: a #DzlSuggestion
+ *
+ * Gets the secondary icon for the suggestion, if any.
+ *
+ * Returns: (transfer full) (nullable): a #GIcon or %NULL
+ *
+ * Since: 3.36
+ */
+GIcon *
+dzl_suggestion_get_secondary_icon (DzlSuggestion *self)
+{
+  g_return_val_if_fail (DZL_IS_SUGGESTION (self), NULL);
+
+  return DZL_SUGGESTION_GET_CLASS (self)->get_secondary_icon (self);
+}
+
+/**
+ * dzl_suggestion_get_secondary_icon_surface:
+ * @self: a #DzlSuggestion
+ * @widget: a widget that may contain the surface
+ *
+ * This function allows subclasses to dynamicly generate content for the
+ * suggestion such as may be required when integrating with favicons or
+ * similar.
+ *
+ * @widget is provided so that the implementation may determine scale or
+ * any other style-specific settings from the style context.
+ *
+ * Returns: (transfer full) (nullable): a #cairo_surface_t or %NULL
+ *
+ * Since: 3.36
+ */
+cairo_surface_t *
+dzl_suggestion_get_secondary_icon_surface (DzlSuggestion *self,
+                                           GtkWidget     *widget)
+{
+  g_return_val_if_fail (DZL_IS_SUGGESTION (self), NULL);
+
+  if (DZL_SUGGESTION_GET_CLASS (self)->get_secondary_icon_surface)
+    return DZL_SUGGESTION_GET_CLASS (self)->get_secondary_icon_surface (self, widget);
+
+  return NULL;
+}
