@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#define _USE_MATH_DEFINES
 #include <math.h>
 
 #include "util/dzl-macros.h"
@@ -199,26 +200,28 @@ dzl_three_grid_get_preferred_width (GtkWidget *widget,
 {
   DzlThreeGrid *self = (DzlThreeGrid *)widget;
   DzlThreeGridPrivate *priv = dzl_three_grid_get_instance_private (self);
-  gint total_min_width = 0;
-  gint total_nat_width = 0;
+  GtkStyleContext *style_context;
+  GtkBorder margin;
   gint min_widths[3];
   gint nat_widths[3];
   gint border_width;
+  GtkStateFlags state;
 
   g_assert (DZL_IS_THREE_GRID (self));
   g_assert (min_width != NULL);
   g_assert (nat_width != NULL);
 
+  style_context = gtk_widget_get_style_context (widget);
+  state = gtk_style_context_get_state (style_context);
+  gtk_style_context_get_margin (style_context, state, &margin);
+
   for (guint i = 0; i < 3; i++)
     dzl_three_grid_get_column_width (self, i, &min_widths[i], &nat_widths[i]);
 
-  total_min_width = MAX (min_widths[0], min_widths[2]) * 2 + min_widths[1];
-  total_nat_width = MAX (nat_widths[0], nat_widths[2]) * 2 + nat_widths[1];
-
   border_width = gtk_container_get_border_width (GTK_CONTAINER (self));
 
-  *min_width = total_min_width + (border_width * 2) + (priv->column_spacing * 2);
-  *nat_width = total_nat_width + (border_width * 2) + (priv->column_spacing * 2);
+  *min_width = MAX (min_widths[0], min_widths[2]) * 2 + min_widths[1] + margin.left + margin.right + (border_width * 2) + (priv->column_spacing * 2);
+  *nat_width = MAX (nat_widths[0], nat_widths[2]) * 2 + nat_widths[1] + margin.left + margin.right + (border_width * 2) + (priv->column_spacing * 2);
 }
 
 static void
@@ -298,7 +301,10 @@ dzl_three_grid_get_preferred_height_for_width (GtkWidget *widget,
   DzlThreeGridPrivate *priv = dzl_three_grid_get_instance_private (self);
   g_autoptr(GHashTable) row_infos = NULL;
   DzlThreeGridRowInfo *row_info;
+  GtkStyleContext *style_context;
   GHashTableIter iter;
+  GtkStateFlags state;
+  GtkBorder margin;
   gint real_min_height = 0;
   gint real_nat_height = 0;
   gint column_min_widths[3];
@@ -315,11 +321,16 @@ dzl_three_grid_get_preferred_height_for_width (GtkWidget *widget,
   width -= border_width * 2;
   width -= priv->column_spacing * 2;
 
+  style_context = gtk_widget_get_style_context (widget);
+  state = gtk_style_context_get_state (style_context);
+  gtk_style_context_get_margin (style_context, state, &margin);
+  width -= margin.left + margin.right;
+
   dzl_three_grid_get_column_width (self, DZL_THREE_GRID_COLUMN_LEFT, &column_min_widths[0], &column_nat_widths[0]);
   dzl_three_grid_get_column_width (self, DZL_THREE_GRID_COLUMN_CENTER, &column_min_widths[1], &column_nat_widths[1]);
   dzl_three_grid_get_column_width (self, DZL_THREE_GRID_COLUMN_RIGHT, &column_min_widths[2], &column_nat_widths[2]);
 
-  if ((MAX (column_min_widths[0], column_min_widths[2]) * 2 + column_nat_widths[1]) >= width)
+  if ((MAX (column_min_widths[0], column_min_widths[2]) * 2 + column_nat_widths[1]) > width)
     {
       widths[0] = column_min_widths[0];
       widths[2] = column_min_widths[2];
@@ -371,6 +382,9 @@ dzl_three_grid_get_preferred_height_for_width (GtkWidget *widget,
   real_min_height += border_width * 2;
   real_nat_height += border_width * 2;
 
+  real_min_height += margin.top + margin.bottom;
+  real_nat_height += margin.top + margin.bottom;
+
   n_rows = g_hash_table_size (row_infos);
 
   if (n_rows > 1)
@@ -435,9 +449,12 @@ dzl_three_grid_size_allocate (GtkWidget     *widget,
   DzlThreeGridPrivate *priv = dzl_three_grid_get_instance_private (self);
   g_autofree GtkRequestedSize *rows = NULL;
   const GList *iter;
+  GtkStyleContext *style_context;
   GtkAllocation area;
   GtkTextDirection dir;
   GList *values;
+  GtkStateFlags state;
+  GtkBorder margin;
   guint i;
   guint n_rows;
   gint min_height;
@@ -456,9 +473,26 @@ dzl_three_grid_size_allocate (GtkWidget     *widget,
   g_assert (DZL_IS_THREE_GRID (self));
   g_assert (allocation != NULL);
 
-  dir = gtk_widget_get_direction (widget);
+  area = *allocation;
 
-  gtk_widget_set_allocation (widget, allocation);
+  style_context = gtk_widget_get_style_context (widget);
+  state = gtk_style_context_get_state (style_context);
+  gtk_style_context_get_margin (style_context, state, &margin);
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (self));
+
+  area.x += border_width;
+  area.y += border_width;
+  area.width -= border_width * 2;
+  area.height -= border_width * 2;
+
+  area.x += margin.left;
+  area.width -= margin.right + margin.left;
+  area.y += margin.top;
+  area.height -= margin.top + margin.bottom;
+
+  gtk_widget_set_allocation (widget, &area);
+
+  dir = gtk_widget_get_direction (widget);
 
   dzl_three_grid_get_preferred_height_for_width (widget, allocation->width, &min_height, &nat_height);
 
@@ -472,14 +506,6 @@ dzl_three_grid_size_allocate (GtkWidget     *widget,
   values = g_hash_table_get_values (priv->row_infos);
   values = g_list_sort (values, sort_by_row);
 
-  area = *allocation;
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (self));
-
-  area.x += border_width;
-  area.y += border_width;
-  area.width -= border_width * 2;
-  area.height -= border_width * 2;
-
   dzl_three_grid_get_column_width (self, DZL_THREE_GRID_COLUMN_LEFT, &left_min_width, &left_nat_width);
   dzl_three_grid_get_column_width (self, DZL_THREE_GRID_COLUMN_CENTER, &center_min_width, &center_nat_width);
   dzl_three_grid_get_column_width (self, DZL_THREE_GRID_COLUMN_RIGHT, &right_min_width, &right_nat_width);
@@ -502,12 +528,11 @@ dzl_three_grid_size_allocate (GtkWidget     *widget,
    * We can handle #1 and #2 with the same logic though.
    */
 
-  if ((MAX (left_min_width, right_min_width) * 2 + center_nat_width) >= (area.width - (gint)(priv->column_spacing * 2)))
+  if ((MAX (left_min_width, right_min_width) * 2 + center_nat_width + 2 * priv->column_spacing) > area.width)
     {
       /* Handle #3 */
-      left = left_min_width;
-      right = right_min_width;
-      center = area.width - left - right;
+      left = right = MAX (left_min_width, right_min_width);
+      center = area.width - left - right - 2 * priv->column_spacing;
     }
   else
     {
@@ -537,7 +562,7 @@ dzl_three_grid_size_allocate (GtkWidget     *widget,
       GtkAllocation child_alloc;
       gint baseline;
 
-      if (row_info->nat_above_baseline + row_info->nat_below_baseline <= size->minimum_size)
+      if (row_info->nat_above_baseline + row_info->nat_below_baseline < size->minimum_size)
         baseline = row_info->nat_above_baseline;
       else
         baseline = row_info->min_above_baseline;
@@ -569,6 +594,8 @@ dzl_three_grid_size_allocate (GtkWidget     *widget,
       area.y += child_alloc.height + priv->row_spacing;
       area.height -= child_alloc.height + priv->row_spacing;
     }
+
+  g_list_free (values);
 }
 
 static void
